// Copyright (c) 2016, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'package:angular2/angular2.dart';

import '../../content/deferred_content.dart';
import '../../material_popup/material_popup.dart';
import '../module.dart';
import '../../../laminate/enums/alignment.dart';
import '../../../laminate/popup/popup.dart' show PopupSource;
import '../../../utils/angular/properties/properties.dart';

import 'tooltip_controller.dart';
import 'tooltip_target.dart';

/// An ink-based overlay meant to deliver contextual information about the
/// element it targets.
///
/// It is a "small" tooltip in that it is one or two lines, at most, and has a
/// maximum width of `160`. Note: add the attribute `twoLine` if your
/// tooltip content is longer than one line.
///
/// Typically, the tooltips are associated with an icon or a button and provide
/// a label or brief helper text about that element. This component should be
/// used with a [TooltipTarget] component which controls its visibility
/// and provides the underlying [ElementRef] at which the tooltip is targeted.
///
/// Use this component in conjunction with the [MaterialTooltipTargetDirective].
///
/// Consider using [MaterialTooltipDirective] instead; usage is much simpler and
/// it enforces text-only small tooltips.
///
/// __Example Usage:__
///
/// ```html
/// <!-- Explicitly declare the tooltip target. Allows for custom content -->
/// <span tooltipTarget #ref="tooltipTarget">
///   Explicitly declare a tooltip component
/// </span>
/// <material-tooltip-text [for]="ref">
///   Allows for <strong>formatted</strong> <em>text</em>.
/// </material-tooltip-text>```
@Component(
    selector: 'material-tooltip-text',
    providers: const [tooltipControllerBinding],
    template: r'''
        <material-popup *ngIf="popupSource != null"
                        ink
                        autoDismiss="false"
                        enforceSpaceConstraints
                        matchSourceWidth="false"
                        matchMinSourceWidth="false"
                        shadowCssClass="aacmtit-ink-tooltip-shadow"
                        trackLayoutChanges
                        [preferredPositions]="positions"
                        [source]="popupSource"
                        [visible]="showPopup">
          <div *deferredContent
               [class.two-line]="twoLine"
               class="ink-container">
            <span>{{text}}<ng-content></ng-content></span>
          </div>
        </material-popup>''',
    styleUrls: const ['ink_tooltip.scss.css'],
    changeDetection: ChangeDetectionStrategy.OnPush,
    directives: const [DeferredContentDirective, NgIf, MaterialPopupComponent])
class MaterialInkTooltipComponent implements Tooltip {
  final TooltipController _tooltipController;
  final ChangeDetectorRef _changeDetector;

  PopupSource _tooltipSource;
  PopupSource get popupSource => _tooltipSource;

  /// Positions at which the tooltip should try to show.
  @Input()
  List<RelativePosition> positions = RelativePosition.AdjacentBottomEdge;

  // Proxy control of this tooltip via the tooltip controller.
  Tooltip _controllerProxy;

  bool _showPopup = false;
  bool get showPopup => _showPopup;

  /// Styles the tooltip to accommodate wrapped tooltip messages.
  @Input()
  set twoLine(b) {
    _twoLine = getBool(b);
  }

  bool _twoLine = false;
  bool get twoLine => _twoLine;

  /// Text content of the tooltip.
  @Input()
  String text;

  MaterialInkTooltipComponent(this._tooltipController, this._changeDetector);

  @override
  void activate() {
    _showPopup = true;
    _changeDetector.markForCheck();
  }

  @override
  void deactivate({bool immediate: false}) {
    _showPopup = false;
    _changeDetector.markForCheck();
  }

  /// The element at which this tooltip is targeted.
  @Input('for')
  set tooltipRef(TooltipTarget target) {
    if (target == null) return;
    _tooltipSource = target;
    target.setTooltip(tooltipHandle);
  }

  Tooltip get tooltipHandle =>
      _controllerProxy ??= _tooltipController.proxyFor(this);
}
